var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _footer, _form;
import createBoundary from "./util/createBoundary.js";
import getMime from "./util/getMimeFromFilename.js";
import isFormData from "./util/isFormData.js";
import isFile from "./util/isFile.js";
const DASHES = "-".repeat(2);
const CRLF = "\r\n";
const CRLF_BYTES_LENGTH = Buffer.byteLength(CRLF);
export class Encoder {
    constructor(form, boundary = createBoundary()) {
        /**
         * Returns field's footer
         */
        _footer.set(this, void 0);
        /**
         * FormData instance
         */
        _form.set(this, void 0);
        if (!isFormData(form)) {
            throw new TypeError("Expected first argument to be a FormData instance.");
        }
        if (typeof boundary !== "string") {
            throw new TypeError("Expected boundary to be a string.");
        }
        this.boundary = boundary;
        this.contentType = `multipart/form-data; boundary=${this.boundary}`;
        __classPrivateFieldSet(this, _form, form);
        __classPrivateFieldSet(this, _footer, `${DASHES}${this.boundary}${DASHES}${CRLF.repeat(2)}`);
    }
    /**
     * Returns headers for multipart/form-data
     */
    get headers() {
        return {
            "Content-Type": this.contentType,
            "Content-Length": this.getContentLength()
        };
    }
    _getFieldHeader(name, value) {
        let header = "";
        header += `${DASHES}${this.boundary}${CRLF}`;
        header += `Content-Disposition: form-data; name="${name}"`;
        if (isFile(value)) {
            header += `; filename="${value.name}"${CRLF}`;
            header += `Content-Type: ${value.type || getMime(value.name)}`;
        }
        return `${header}${CRLF.repeat(2)}`;
    }
    /**
     * Returns form-data content length
     */
    getContentLength() {
        let length = 0;
        for (const [name, value] of __classPrivateFieldGet(this, _form)) {
            length += Buffer.byteLength(this._getFieldHeader(name, value));
            length += isFile(value) ? value.size : Buffer.byteLength(String(value));
            length += CRLF_BYTES_LENGTH;
        }
        return length + Buffer.byteLength(__classPrivateFieldGet(this, _footer));
    }
    async *_getField() {
        for (const [name, value] of __classPrivateFieldGet(this, _form)) {
            yield this._getFieldHeader(name, value);
            if (isFile(value)) {
                yield* value.stream();
            }
            else {
                yield value;
            }
            yield CRLF;
        }
        yield __classPrivateFieldGet(this, _footer);
    }
    /**
     * Returns async generator allowing to encode FormData content into the spec format
     */
    async *encode() {
        for await (const chunk of this._getField()) {
            yield Buffer.isBuffer(chunk) ? chunk : Buffer.from(String(chunk));
        }
    }
    [(_footer = new WeakMap(), _form = new WeakMap(), Symbol.asyncIterator)]() {
        return this.encode();
    }
}
